/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: controller.c,v 1.15 2004/07/28 21:15:42 cheako Exp $
 *
 * Description: This file handles joystick input.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include "globals.h"
#include "renderer.h"
#include "controller.h"

#include "controller_x11.h"
#include "controller_linux.h"
#include "controller_sdl.h"
#include "emulator.h"

struct Controller controllers[] = {
#ifdef HAVE_LINUX_JOYSTICK_H
	{ "linux", "Linux joystik driver", {0, 0}, {0, 0}, 0, 0, InitJoystickLinux, 
	  HandleJoystickLinux, ShutdownJoystickLinux },
#endif
#ifdef HAVE_X
	{ "x11", "X11 driver", {0, 0}, {0, 0}, 0, 0, InitJoystickX11, 
	  HandleJoystickX11, ShutdownJoystickX11 },
#endif
#ifdef HAVE_LIBSDL
	{ "sdl", "SDL driver", {0, 0}, {0, 0}, 0, 0, InitJoystickSDL, 
	  HandleJoystickSDL, ShutdownJoystickSDL },
#endif /* HAVE_LIBSDL */
	{ 0, 0, { 0, 0 }, { 0, 0}, 0, 0, 0, 0, 0 }
}, *controller = 0;
