/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: controller_linux.c,v 1.4 2004/07/28 21:15:42 cheako Exp $
 *
 * Description: This file handles joystick input for linux joystick.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include "globals.h"
#include "renderer.h"
#include "controller.h"
#include "controller_linux.h"
#include "controller_x11.h"
#include "emulator.h"

#ifdef HAVE_LINUX_JOYSTICK_H
char jsaxes[2] = { 2, 2 };
char jsbuttons[2] = { 2, 2 };
int jsfd[2] = { -1, -1 };
int jsversion[2] = { 0x000800, 0x000800 };

/* ============================ InitJoystickLinux () ======================= */
void InitJoystickLinux(int stick)
{
    if ((jsfd[stick] = open(jsdevice[stick], O_RDONLY)) >= 0) {
	ioctl(jsfd[stick], JSIOCGVERSION, jsversion + stick);
	ioctl(jsfd[stick], JSIOCGAXES, jsaxes + stick);
	ioctl(jsfd[stick], JSIOCGBUTTONS, jsbuttons + stick);
	fcntl(jsfd[stick], F_SETFL, O_NONBLOCK);	/* set joystick to non-blocking */
	if (jsbuttons[stick] == 2) {	/* modify button map for a 2-button joystick */
	    if (!js_usermapped2button[stick][0])
		js_nesmaps[stick].button[0] = BUTTONB;
	    if (!js_usermapped2button[stick][1])
		js_nesmaps[stick].button[1] = BUTTONA;
	}
	if (verbose) {
	    fprintf(stderr,
		    "Joystick %d (%s) has %d axes and %d buttons. Driver version is %d.%d.%d.\n",
		    stick + 1,
		    jsdevice[stick],
		    jsaxes[stick],
		    jsbuttons[stick],
		    jsversion[stick] >> 16,
		    (jsversion[stick] >> 8) & 0xff,
		    jsversion[stick] & 0xff);
	}
    }
#ifdef HAVE_X
   if ( renderer->name == "x11" || renderer->name == "old" || renderer->name == "diff" )
     InitJoystickX11(stick);
#endif
}

/* ============================ HandleJoystickLinux () ===================== */
void HandleJoystickLinux(int stick)
{
    int nes_button;
    int axis_i;
    struct js_event js;
if ( jsfd[stick] != -1 ) {
    while (read(jsfd[stick], &js, sizeof(struct js_event)) ==
	   sizeof(struct js_event)) {
	/* verbose joystick reporting */
/*        if (verbose) */
/*  	{ */
/*  	  fprintf (stderr, "%s: type %d number %d value %d\n", */
/*  		   jsdevice[stick], js.type, js.number, js.value); */
/*  	} */
	switch (js.type) {
	case 1:		/* button report */
	    nes_button = js_nesmaps[stick].button[js.number];
	    if (nes_button != PAUSEDISPLAY) {
		if (js.value)
		    controller->joypad[stick] |= nes_button;
		else
		    controller->joypad[stick] &= ~nes_button;
	    } else /* (nes_button == PAUSEDISPLAY) */ if (js.value) {
		renderer_data.pause_display = !renderer_data.pause_display;
		desync = 1;
	    }
	    break;
	case 2:		/* axis report */
	    if (js.value < -JS_IGNORE) {
		js_axismeso[stick][js.number] =
		    js_nesmaps[stick].axis[js.number].neg;
		controller->joypad[stick] &=
		    ~js_nesmaps[stick].axis[js.number].pos;
	    } else if (js.value > JS_IGNORE) {
		js_axismeso[stick][js.number] =
		    js_nesmaps[stick].axis[js.number].pos;
		controller->joypad[stick] &=
		    ~js_nesmaps[stick].axis[js.number].neg;
	    } else {
		js_axismeso[stick][js.number] = 0;
		controller->joypad[stick] &=
		    ~js_nesmaps[stick].axis[js.number].neg;
		controller->joypad[stick] &=
		    ~js_nesmaps[stick].axis[js.number].pos;
	    }
	    for (axis_i = JS_MAX_AXES; --axis_i >= 0;)
		controller->joypad[stick] |= js_axismeso[stick][axis_i];
	    break;
	}
    }
    if (errno != EAGAIN) {
	if (errno) {
	    perror(jsdevice[stick]);
	} else {
	    fprintf(stderr,
		    "%s: device violates joystick protocol, disabling\n",
		    jsdevice[stick]);
	    close(jsfd[stick]);
	    jsfd[stick] = -1;
	}
    }
}
#ifdef HAVE_X
   if ( renderer->name == "x11" || renderer->name == "old" || renderer->name == "diff" )
     HandleJoystickX11(stick);
#endif
}


/* ============================ ShutdownJoystickLinux () =================== */ 
void ShutdownJoystickLinux(int stick)
{
   close(jsfd[stick]);
#ifdef HAVE_X
   if ( renderer->name == "x11" || renderer->name == "old" || renderer->name == "diff" )
     ShutdownJoystickX11(stick);
#endif
}
#endif /* HAVE_LINUX_JOYSTICK_H */
