/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: controller_sdl.c,v 1.8 2004/07/28 21:15:42 cheako Exp $
 *
 * Description: This file handles the I/O subsystem when using SDL.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_FEATURES_H
#include <features.h>
#endif				/* HAVE_FEATRES_H */

#include <stdlib.h>
#include "globals.h"
#include "renderer.h"
#include "controller.h"
#include "controller_sdl.h"
#include "emulator.h"

#ifdef HAVE_LIBSDL

#include <SDL/SDL.h>

SDL_Joystick *sdl_js[2];

/* ============================ InitJoystickSDL () ========================= */
void InitJoystickSDL(int stick)
{
    if (SDL_Init(SDL_INIT_JOYSTICK) < 0) {
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
	exit(1);
    }
    SDL_JoystickEventState(SDL_ENABLE);
    printf ("stick: %d baz\n", stick);
    if (! (sdl_js[stick] = SDL_JoystickOpen(stick)))
        fprintf (stderr, "Error initializing joystick %d\n", stick);;
}

/* ============================ handle_key () ============================== */
static void handle_key(SDL_KeyboardEvent * ev)
{
    if (ev->keysym.sym == SDLK_ESCAPE)
	quit();
    /* the coin and dipswitch inputs work only in VS UniSystem mode */
    if (unisystem)
	switch (ev->keysym.sym) {
	case SDLK_LEFTBRACKET:
	    if (renderer_config.sticky_keys) {
		if (ev->type == SDL_KEYDOWN)
		    controller->coinslot ^= 1;
	    } else if (ev->type == SDL_KEYDOWN)
		controller->coinslot |= 1;
	    else
		controller->coinslot &= ~1;
	    break;
	case SDLK_RIGHTBRACKET:
	    if (renderer_config.sticky_keys) {
		if (ev->type == SDL_KEYDOWN)
		    controller->coinslot ^= 2;
	    } else if (ev->type == SDL_KEYDOWN)
		controller->coinslot |= 2;
	    else
		controller->coinslot &= ~2;
	    break;
	case SDLK_BACKSLASH:
	    if (renderer_config.sticky_keys) {
		if (ev->type == SDL_KEYDOWN)
		    controller->coinslot ^= 4;
	    } else if (ev->type == SDL_KEYDOWN)
		controller->coinslot |= 4;
	    else
		controller->coinslot &= ~4;
	    break;
	case SDLK_q:
	    if (ev->type == SDL_KEYDOWN)
		controller->dipswitches ^= 0x01;
	    break;
	case SDLK_w:
	    if (ev->type == SDL_KEYDOWN)
		controller->dipswitches ^= 0x02;
	    break;
	case SDLK_e:
	    if (ev->type == SDL_KEYDOWN)
		controller->dipswitches ^= 0x04;
	    break;
	case SDLK_r:
	    if (ev->type == SDL_KEYDOWN)
		controller->dipswitches ^= 0x08;
	    break;
	case SDLK_t:
	    if (ev->type == SDL_KEYDOWN)
		controller->dipswitches ^= 0x10;
	    break;
	case SDLK_y:
	    if (ev->type == SDL_KEYDOWN)
		controller->dipswitches ^= 0x20;
	    break;
	case SDLK_u:
	    if (ev->type == SDL_KEYDOWN)
		controller->dipswitches ^= 0x40;
	    break;
	case SDLK_i:
	    if (ev->type == SDL_KEYDOWN)
		controller->dipswitches ^= 0x80;
	    break;
	default:
	    break;
	}
    switch (ev->keysym.sym) {
	/* controller 1 keyboard mapping */
    case SDLK_RETURN:
    case SDLK_KP_ENTER:
	if (renderer_config.sticky_keys) {
	    if (ev->type == SDL_KEYDOWN)
		controller->joypad[0] ^= STARTBUTTON;
	} else if (ev->type == SDL_KEYDOWN)
	    controller->joypad[0] |= STARTBUTTON;
	else
	    controller->joypad[0] &= ~STARTBUTTON;
	break;
    case SDLK_TAB:
	if (renderer_config.sticky_keys) {
	    if (ev->type == SDL_KEYDOWN)
		controller->joypad[0] ^= SELECTBUTTON;
	} else if (ev->type == SDL_KEYDOWN)
	    controller->joypad[0] |= SELECTBUTTON;
	else
	    controller->joypad[0] &= ~SELECTBUTTON;
	break;
    case SDLK_UP:
	if (renderer_config.sticky_keys) {
	    if (ev->type == SDL_KEYDOWN)
		controller->joypad[0] ^= UP;
	} else if (ev->type == SDL_KEYDOWN)
	    controller->joypad[0] |= UP;
	else
	    controller->joypad[0] &= ~UP;
	break;
    case SDLK_DOWN:
	if (renderer_config.sticky_keys) {
	    if (ev->type == SDL_KEYDOWN)
		controller->joypad[0] ^= DOWN;
	} else if (ev->type == SDL_KEYDOWN)
	    controller->joypad[0] |= DOWN;
	else
	    controller->joypad[0] &= ~DOWN;
	break;
    case SDLK_LEFT:
    case SDLK_HOME:
    case SDLK_END:
	if (renderer_config.sticky_keys) {
	    if (ev->type == SDL_KEYDOWN)
		controller->joypad[0] ^= LEFT;
	} else if (ev->type == SDL_KEYDOWN)
	    controller->joypad[0] |= LEFT;
	else
	    controller->joypad[0] &= ~LEFT;
	break;
    case SDLK_RIGHT:
    case SDLK_PAGEUP:
    case SDLK_PAGEDOWN:
	if (renderer_config.sticky_keys) {
	    if (ev->type == SDL_KEYDOWN)
		controller->joypad[0] ^= RIGHT;
	} else if (ev->type == SDL_KEYDOWN)
	    controller->joypad[0] |= RIGHT;
	else
	    controller->joypad[0] &= ~RIGHT;
	break;
    case SDLK_z:
    case SDLK_x:
    case SDLK_d:
    case SDLK_INSERT:
	if (renderer_config.sticky_keys) {
	    if (ev->type == SDL_KEYDOWN)
		controller->joypad[0] ^= BUTTONB;
	} else if (ev->type == SDL_KEYDOWN)
	    controller->joypad[0] |= BUTTONB;
	else
	    controller->joypad[0] &= ~BUTTONB;
	break;
    case SDLK_a:
    case SDLK_c:
    case SDLK_SPACE:
    case SDLK_DELETE:
	if (renderer_config.sticky_keys) {
	    if (ev->type == SDL_KEYDOWN)
		controller->joypad[0] ^= BUTTONA;
	} else if (ev->type == SDL_KEYDOWN)
	    controller->joypad[0] |= BUTTONA;
	else
	    controller->joypad[0] &= ~BUTTONA;
	break;
    case SDLK_F7:
    case SDLK_PRINT:
    case SDLK_s:
	renderer->SaveScreenshot();
	break;
    case SDLK_F10:
    	if (renderer->ToggleFullScreen)
	    renderer->ToggleFullScreen ();
        break;
    default:
	break;
    }
}

/* ============================ HandleJoystickSDL () ======================= */
void HandleJoystickSDL(int stick)
{
    /* Handles all input, actually. */
    SDL_Event event;
    int axis_i;
    while (SDL_PollEvent(&event)) {
	switch (event.type) {
	    int nes_button;
	case SDL_KEYDOWN:
	case SDL_KEYUP:
	    handle_key(&event.key);
	    break;
	case SDL_QUIT:
	    quit();
	case SDL_JOYAXISMOTION:
	    /* Left-right movement */
	    if (event.jaxis.value < -JS_IGNORE) {
		js_axismeso[stick][event.jaxis.axis] =
		    js_nesmaps[stick].axis[event.jaxis.axis].neg;
		controller->joypad[stick] &=
		    ~js_nesmaps[stick].axis[event.jaxis.axis].pos;
	    } else if (event.jaxis.value > JS_IGNORE) {
		js_axismeso[stick][event.jaxis.axis] =
		    js_nesmaps[stick].axis[event.jaxis.axis].pos;
		controller->joypad[stick] &=
		    ~js_nesmaps[stick].axis[event.jaxis.axis].neg;
	    } else {
		js_axismeso[stick][event.jaxis.axis] = 0;
		controller->joypad[stick] &=
		    ~js_nesmaps[stick].axis[event.jaxis.axis].neg;
		controller->joypad[stick] &=
		    ~js_nesmaps[stick].axis[event.jaxis.axis].pos;
	    }
	    for (axis_i = JS_MAX_AXES; --axis_i >= 0;)
		controller->joypad[stick] |= js_axismeso[stick][axis_i];
	    break;
	case SDL_JOYBUTTONDOWN:
	    nes_button = js_nesmaps[stick].button[event.jbutton.button];
	    if (nes_button != PAUSEDISPLAY) {
		controller->joypad[stick] |= nes_button;
	    } else {
		renderer_data.pause_display = !renderer_data.pause_display;
		desync = 1;
	    }
	    break;
	case SDL_JOYBUTTONUP:
	    nes_button = js_nesmaps[stick].button[event.jbutton.button];
	    if (nes_button != PAUSEDISPLAY) {
		controller->joypad[stick] &= ~nes_button;
	    } else {
		renderer_data.pause_display = !renderer_data.pause_display;
		desync = 1;
	    }
	    break;
	}
    }
}

/* ============================ ShutdownJoystickSDL () ===================== */
void ShutdownJoystickSDL(int stick)
{
    SDL_JoystickClose(sdl_js[stick]);
}

#endif				/* HAVE_LIBSDL */
