/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: controller_x11.c,v 1.2 2004/07/28 21:15:42 cheako Exp $
 *
 * Description: This file handles joystick input for X11.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_FEATURES_H
#include <features.h>
#endif				/* HAVE_FEATRES_H */

#include <stdio.h>
#include <string.h>
#include "globals.h"
#include "renderer.h"
#include "controller.h"
#include "controller_x11.h"
#include "emulator.h"

#ifdef HAVE_X

/* ============================ InitJoystickLinux () ======================= */
void InitJoystickX11(int stick)
{
   return; /* Fixme move/validate code from render_x11.c */
}

/* ============================ HandleKeyboardX11 () ======================= */ 
static void HandleKeyboardX11(XEvent ev)
{
  KeySym keysym = XKeycodeToKeysym (display, ((XKeyEvent *) & ev)->keycode, 0);
  if (ev.type == KeyPress && keysym == XK_Escape)
    quit ();                    /* ESC */
  /* the coin and dipswitch inputs work only in VS UniSystem mode */
  if (unisystem)
    switch (keysym)
      {
      case XK_bracketleft:
      case XK_braceleft:
        if (renderer_config.sticky_keys)
          {
            if (ev.type == KeyPress)
              controller->coinslot ^= 1;
          }
        else if (ev.type == KeyPress)
          controller->coinslot |= 1;
        else
          controller->coinslot &= ~1;
        break;
      case XK_bracketright:
      case XK_braceright:
        if (renderer_config.sticky_keys)
          {
            if (ev.type == KeyPress)
              controller->coinslot ^= 2;
          }
        else if (ev.type == KeyPress)
          controller->coinslot |= 2;
        else
          controller->coinslot &= ~2;
        break;
      case XK_backslash:
      case XK_bar:
        if (renderer_config.sticky_keys)
          {
            if (ev.type == KeyPress)
              controller->coinslot ^= 4;
          }
        else if (ev.type == KeyPress)
          controller->coinslot |= 4;
        else
          controller->coinslot &= ~4;
        break;
      case XK_Q:
      case XK_q:
        if (ev.type == KeyPress)
          controller->dipswitches ^= 0x01;
        break;
      case XK_W:
      case XK_w:
        if (ev.type == KeyPress)
          controller->dipswitches ^= 0x02;
        break;
      case XK_E:
      case XK_e:
        if (ev.type == KeyPress)
          controller->dipswitches ^= 0x04;
        break;
      case XK_R:
      case XK_r:
        if (ev.type == KeyPress)
          controller->dipswitches ^= 0x08;
        break;
      case XK_T:
      case XK_t:
        if (ev.type == KeyPress)
          controller->dipswitches ^= 0x10;
        break;
      case XK_Y:
      case XK_y:
        if (ev.type == KeyPress)
          controller->dipswitches ^= 0x20;
        break;
      case XK_U:
      case XK_u:
        if (ev.type == KeyPress)
          controller->dipswitches ^= 0x40;
        break;
      case XK_I:
      case XK_i:
        if (ev.type == KeyPress)
          controller->dipswitches ^= 0x80;
        break;
      }
  switch (keysym)
    {
      /* controller 1 keyboard mapping */
    case XK_Return:
    case XK_KP_Enter:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[0] ^= STARTBUTTON;
        }
      else if (ev.type == KeyPress)
        controller->joypad[0] |= STARTBUTTON;
      else
        controller->joypad[0] &= ~STARTBUTTON;
      break;
    case XK_Tab:
    case XK_ISO_Left_Tab:
    case XK_KP_Add:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[0] ^= SELECTBUTTON;
        }
      else if (ev.type == KeyPress)
        controller->joypad[0] |= SELECTBUTTON;
      else
        controller->joypad[0] &= ~SELECTBUTTON;
      break;
    case XK_Up:
    case XK_KP_Up:
    case XK_KP_8:
    case XK_Pointer_Up:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[0] ^= UP;
        }
      else if (ev.type == KeyPress)
        controller->joypad[0] |= UP;
      else
        controller->joypad[0] &= ~UP;
      break;
    case XK_Down:
    case XK_KP_Down:
    case XK_KP_2:
    case XK_Pointer_Down:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[0] ^= DOWN;
        }
      else if (ev.type == KeyPress)
        controller->joypad[0] |= DOWN;
      else
        controller->joypad[0] &= ~DOWN;
      break;
    case XK_Left:
    case XK_KP_Left:
    case XK_KP_4:
    case XK_Pointer_Left:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[0] ^= LEFT;
        }
      else if (ev.type == KeyPress)
        controller->joypad[0] |= LEFT;
      else
        controller->joypad[0] &= ~LEFT;
      break;
    case XK_Right:
    case XK_KP_Right:
    case XK_KP_6:
    case XK_Pointer_Right:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[0] ^= RIGHT;
        }
      else if (ev.type == KeyPress)
        controller->joypad[0] |= RIGHT;
      else
        controller->joypad[0] &= ~RIGHT;
      break;
    case XK_Pointer_UpLeft:
    case XK_Home:
    case XK_KP_Home:
    case XK_KP_7:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypadd[0] ^= (UP|LEFT);
        }
      else if (ev.type == KeyPress)
        controller->joypadd[0] |= (UP|LEFT);
      else
        controller->joypadd[0] &= ~(UP|LEFT);
      break;
    case XK_Pointer_UpRight:
    case XK_Prior:
    case XK_KP_Prior:
    case XK_KP_9:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypadd[0] ^= (UP|RIGHT);
        }
      else if (ev.type == KeyPress)
        controller->joypadd[0] |= (UP|RIGHT);
      else
        controller->joypadd[0] &= ~(UP|RIGHT);
      break;
    case XK_Pointer_DownLeft:
    case XK_End:
    case XK_KP_End:
    case XK_KP_1:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypadd[0] ^= (DOWN|LEFT);
        }
      else if (ev.type == KeyPress)
        controller->joypadd[0] |= (DOWN|LEFT);
      else
        controller->joypadd[0] &= ~(DOWN|LEFT);
      break;
    case XK_Pointer_DownRight:
    case XK_Next:
    case XK_KP_Next:
    case XK_KP_3:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypadd[0] ^= (DOWN|RIGHT);
        }
      else if (ev.type == KeyPress)
        controller->joypadd[0] |= (DOWN|RIGHT);
      else
        controller->joypadd[0] &= ~(DOWN|RIGHT);
      break;
    case XK_Z:
    case XK_z:
    case XK_X:
    case XK_x:
    case XK_D:
    case XK_d:
    case XK_Insert:
    case XK_KP_Insert:
    case XK_KP_0:
    case XK_Shift_L:
    case XK_Shift_R:
    case XK_Eisu_Shift:
    case XK_Control_L:
    case XK_Alt_R:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[0] ^= BUTTONB;
        }
      else if (ev.type == KeyPress)
        controller->joypad[0] |= BUTTONB;
      else
        controller->joypad[0] &= ~BUTTONB;
      break;
    case XK_A:
    case XK_a:
    case XK_C:
    case XK_c:
    case XK_space:
    case XK_KP_Space:
    case XK_Clear:
    case XK_Delete:
    case XK_KP_Delete:
    case XK_KP_Begin:
    case XK_KP_5:
    case XK_KP_Decimal:
    case XK_Alt_L:
    case XK_Control_R:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[0] ^= BUTTONA;
        }
      else if (ev.type == KeyPress)
        controller->joypad[0] |= BUTTONA;
      else
        controller->joypad[0] &= ~BUTTONA;
      break;
      /* controller 2 keyboard mapping */
    case XK_G:
    case XK_g:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[1] ^= STARTBUTTON;
        }
      else if (ev.type == KeyPress)
        controller->joypad[1] |= STARTBUTTON;
      else
        controller->joypad[1] &= ~STARTBUTTON;
      break;
    case XK_F:
    case XK_f:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[1] ^= SELECTBUTTON;
        }
      else if (ev.type == KeyPress)
        controller->joypad[1] |= SELECTBUTTON;
      else
        controller->joypad[1] &= ~SELECTBUTTON;
      break;
    case XK_K:
    case XK_k:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[1] ^= UP;
        }
      else if (ev.type == KeyPress)
        controller->joypad[1] |= UP;
      else
        controller->joypad[1] &= ~UP;
      break;
    case XK_J:
    case XK_j:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[1] ^= DOWN;
        }
      else if (ev.type == KeyPress)
        controller->joypad[1] |= DOWN;
      else
        controller->joypad[1] &= ~DOWN;
      break;
    case XK_H:
    case XK_h:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[1] ^= LEFT;
        }
      else if (ev.type == KeyPress)
        controller->joypad[1] |= LEFT;
      else
        controller->joypad[1] &= ~LEFT;
      break;
    case XK_L:
    case XK_l:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[1] ^= RIGHT;
        }
      else if (ev.type == KeyPress)
        controller->joypad[1] |= RIGHT;
      else
        controller->joypad[1] &= ~RIGHT;
      break;
    case XK_V:
    case XK_v:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[1] ^= BUTTONB;
        }
      else if (ev.type == KeyPress)
        controller->joypad[1] |= BUTTONB;
      else
        controller->joypad[1] &= ~BUTTONB;
      break;
    case XK_B:
    case XK_b:
      if (renderer_config.sticky_keys)
        {
          if (ev.type == KeyPress)
            controller->joypad[1] ^= BUTTONA;
        }
      else if (ev.type == KeyPress)
        controller->joypad[1] |= BUTTONA;
      else
        controller->joypad[1] &= ~BUTTONA;
      break;
    }
  /* emulator keys */
  if (ev.type == KeyPress)
    switch (keysym)
      {
      case XK_F1:
        debug_bgoff = 1;
        break;
      case XK_F2:
        debug_bgoff = 0;
        break;
      case XK_F3:
        debug_spritesoff = 1;
        break;
      case XK_F4:
        debug_spritesoff = 0;
        break;
      case XK_F5:
        debug_switchtable = 1;
        break;
      case XK_F6:
        debug_switchtable = 0;
        break;
      case XK_F7:
      case XK_Print:
      case XK_S:
      case XK_s:
        renderer->SaveScreenshot ();
        break;
      case XK_F8:
        memset (displaycachevalid, 0, sizeof(displaycachevalid));
	memset (bitplanecachevalid, 0, sizeof(bitplanecachevalid));
        break;
      case XK_F10:
        if (renderer->ToggleFullScreen)
	  renderer->ToggleFullScreen ();
        break;
      case XK_BackSpace:
        emulator->Start();
        break;
      case XK_Pause:
      case XK_P:
      case XK_p:
	renderer_data.pause_display = ! renderer_data.pause_display;
	desync = 1;
	break;
      case XK_grave:
        desync = 1;
        halfspeed = 1;
        doublespeed = 0;
        renderer_data.pause_display = 0;
        break;
      case XK_1:
        desync = 1;
        halfspeed = 0;
        doublespeed = 0;
        renderer_data.pause_display = 0;
        break;
      case XK_2:
        desync = 1;
        halfspeed = 0;
        doublespeed = 2;
        renderer_data.pause_display = 0;
        break;
      case XK_3:
        desync = 1;
        halfspeed = 0;
        doublespeed = 3;
        renderer_data.pause_display = 0;
        break;
      case XK_4:
        desync = 1;
        halfspeed = 0;
        doublespeed = 4;
        renderer_data.pause_display = 0;
        break;
      case XK_5:
        desync = 1;
        halfspeed = 0;
        doublespeed = 5;
        renderer_data.pause_display = 0;
        break;
      case XK_6:
        desync = 1;
        halfspeed = 0;
        doublespeed = 6;
        renderer_data.pause_display = 0;
        break;
      case XK_7:
        desync = 1;
        halfspeed = 0;
        doublespeed = 7;
        renderer_data.pause_display = 0;
        break;
      case XK_8:
        desync = 1;
        halfspeed = 0;
        doublespeed = 8;
        renderer_data.pause_display = 0;
        break;
      case XK_0:
        desync = 1;
        renderer_data.pause_display = 1;
        break;
      }
}

void HandleJoystickX11(int stick)
{
    static XEvent ev;
if (renderer->_flags & RENDERER_NON_X11) return;

    /* Handle X input */
    while (XPending (display) || ev.type == -1)
      {
	XNextEvent (display, &ev);
	/*printf("event %d\n",ev.type); */
	if (ev.type == DestroyNotify)
	  quit ();
	/* if you'd like defocusing to pause emulation, define this */
#ifdef PAUSE_ON_DEFOCUS
	if (ev.type == FocusIn)
	  renderer_data.pause_program = 0;
	if (ev.type == FocusOut)
	  renderer_data.pause_program = 1;
#endif
	if (ev.type == MapNotify)
	  {
	    XExposeEvent *xev = (XExposeEvent *) &ev;
	    xev -> type = Expose;
	    xev -> count = 0;
	    /* FIXME nodisplay = 0;
	    */
	    needsredraw = redrawall = 1;
	  }
	else if (ev.type == UnmapNotify)
	  {
	    /* FIXME nodisplay = 1;
	    */
	    needsredraw = redrawall = 0;
	  }
	/* Handle keyboard input */
	if (ev.type == KeyPress || ev.type == KeyRelease)
	  {
	    HandleKeyboardX11 (ev);
	  }
	if (ev.type == KeymapNotify)
	  {
	    memcpy (keystate, ((XKeymapEvent *) & ev)->key_vector, 32);
	    controller->joypad[0] = controller->joypad[1] = 0;
	    controller->joypadd[0] = controller->joypadd[1] = 0;
	  }
	if (ev.type == ConfigureNotify)
	  {
	    XConfigureEvent *ce = (XConfigureEvent *)&ev;
	    if ((ce -> width != width) ||
		(ce -> height != height))
	      {
		width = ce -> width;
		height = ce -> height;
		XFillRectangle (display, w,
				blackgc, 0, 0,
				width, height);
	      }
	  }
	if (ev.type == Expose)
	  {
	    XExposeEvent *xev = (XExposeEvent *) &ev;
	    if (! xev -> count)
	      {
#if HAVE_SHM
		if (shm_attached)
		  {
		    XShmPutImage (display, w, gc, image, 0, 0,
				  (256 * magstep - width) / -2,
				  (240 * magstep - height) / -2,
				  256 * magstep, 240 * magstep,
				  True);
		    /* hang the event loop until we get a ShmCompletion */
		    ev.type = -1;
		  }
		else
#endif
		  {
		    XPutImage (display, w, gc, image, 0, 0,
			       (256 * magstep - width) / -2,
			       (240 * magstep - height) / -2,
			       256 * magstep, 240 * magstep);
		    XFlush (display);
		  }
	      }
	  }
	if (PAUSED && needsredraw)
	  {
	    XCopyArea (display, layout, w, gc,
		       0, 0,
		       256 * magstep, 240 * magstep,
		       (256 * magstep - width) / -2,
		       (240 * magstep - height) / -2);
	    needsredraw = 0;
	  }
      }
}

void ShutdownJoystickX11(int stick)
{
   return;
}

#endif /* HAVE_X */
