/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: emulator_x86.c,v 1.12 2004/07/28 21:15:42 cheako Exp $
 *
 * Description: C Code for the X86asm routines.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_X86

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include "globals.h"
#include "mapper.h"
#include "emulator.h"
#include "emulator_x86.h"
#include "renderer.h"
#include "controller.h"

static int last_clock = 0; /* For vblank bit */

extern int vbl;
extern int hscrollval, vscrollval;

extern void MAPPER_NONE;
extern void MAPPER_MMC1;
extern void MAPPER_UNROM;
extern void MAPPER_CNROM;
extern void MAPPER_MMC3;
extern void MAPPER_MMC5;
extern void MAPPER_MMC2;
extern void MAPPER_MMC4;
extern void MAPPER_CLRDRMS;
extern void MAPPER_AOROM;
extern void MAPPER_CPROM;
extern void MAPPER_100IN1;
extern void MAPPER_NAMCOT106;
extern void MAPPER_VRC2_A;
extern void MAPPER_VRC2_B;
extern void MAPPER_G101;
extern void MAPPER_TAITO_TC0190;
extern void MAPPER_GNROM;
extern void MAPPER_TENGEN_RAMBO1;
extern void MAPPER_SUNSOFT4;
extern void MAPPER_FME7;
extern void MAPPER_CAMERICA;
extern void MAPPER_IREM_74HC161_32;
extern void MAPPER_VS;
extern void MAPPER_SUPERVISION;
extern void MAPPER_NINA7;

void StartX86 (void)
{
    STACKPTR = (int) STACK + 0xFF;
    translate (*((unsigned short *) (MAPTABLE[15] + 0xFFFC)));
    START ();
}

/* This determines whether an NMI should occur and returns 1 if so. */
/* This function is called even when interrupts are off, and also */
/* refreshes the screen as necessary. */
int DoNMIX86(void)
{
    int x;
    /*printf("donmi: at %d\n",CLOCK); */
    CLOCK = VBL + 7;		/* 7 cycle interrupt latency */
    CTNI = -CPF + 7;
    if (last_clock >= CLOCK || last_clock <= 27393)
	vbl = 1;
    last_clock = CLOCK;
    renderer->UpdateDisplay();
    controller->HandleJoystick(0);
    controller->HandleJoystick(1);
    /*printf("donmi: stack at %x\n",STACKPTR); */
    /* reset scroll registers */
    /*hvscroll = 0; */
    for (x = 0; x < 240; x++) {
	hscroll[x] = hscrollval;
	vscroll[x] = vscrollval;
	linereg[x] = emulator->RAM[0x2000];
    }
    /* Is an NMI to be generated?  Return 1 if so. */
    if (((*((unsigned char *) (emulator->RAM+0x2000))) & 0x80) != 0)
	return 1;
    else
	return 0;
}

/* storage space */
void (*MapperInit_X86[MAXMAPPER + 1])(void);
void *Mapper_X86[MAXMAPPER + 1];

void InitMapperSubsystemX86(void)
{
    int x;
    /* clear them all to zero */
    for (x = 0; x < MAXMAPPER; x++) {
	emulator->MapperInit[x] = 0;
	emulator->Mapper[x] = 0;
	MapperName[x][0] = '\0';
    }
    emulator->MapperInit[0] = &init_none;
    emulator->Mapper[0] = &MAPPER_NONE;
    strcpy(MapperName[0], "No mapper");
    emulator->MapperInit[1] = &init_mmc1;
    emulator->Mapper[1] = &MAPPER_MMC1;
    strcpy(MapperName[1], "MMC1");
    emulator->MapperInit[2] = &init_unrom;
    emulator->Mapper[2] = &MAPPER_UNROM;
    strcpy(MapperName[2], "UNROM");
    emulator->MapperInit[3] = &init_cnrom;
    emulator->Mapper[3] = &MAPPER_CNROM;
    strcpy(MapperName[3], "CNROM");
    emulator->MapperInit[4] = &init_mmc3;
    emulator->Mapper[4] = &MAPPER_MMC3;
    strcpy(MapperName[4], "MMC3");
    emulator->MapperInit[5] = &init_mmc5;
    emulator->Mapper[5] = &MAPPER_MMC5;
    strcpy(MapperName[5], "MMC5");
    emulator->MapperInit[7] = &init_aorom;
    emulator->Mapper[7] = &MAPPER_AOROM;
    strcpy(MapperName[7], "AOROM");
    emulator->MapperInit[9] = &init_mmc2;
    emulator->Mapper[9] = &MAPPER_MMC2;
    strcpy(MapperName[9], "MMC2");
    emulator->MapperInit[10] = &init_mmc4;
    emulator->Mapper[10] = &MAPPER_MMC4;
    strcpy(MapperName[10], "MMC4");
    emulator->MapperInit[11] = &init_clrdrms;
    emulator->Mapper[11] = &MAPPER_CLRDRMS;
    strcpy(MapperName[11], "Color Dreams");
    emulator->MapperInit[13] = &init_cprom;
    emulator->Mapper[13] = &MAPPER_CPROM;
    strcpy(MapperName[13], "CPROM");
    emulator->MapperInit[15] = &init_100in1;
    emulator->Mapper[15] = &MAPPER_100IN1;
    strcpy(MapperName[15], "100-in-1");
    emulator->MapperInit[19] = &init_namcot106;
    emulator->Mapper[19] = &MAPPER_NAMCOT106;
    strcpy(MapperName[19], "Namcot 106");
    emulator->MapperInit[22] = &init_vrc2_a;
    emulator->Mapper[22] = &MAPPER_VRC2_A;
    strcpy(MapperName[22], "Konami VRC2 type A");
    emulator->MapperInit[23] = &init_vrc2_b;
    emulator->Mapper[23] = &MAPPER_VRC2_B;
    strcpy(MapperName[23], "Konami VRC2 type B");
    emulator->MapperInit[32] = &init_g101;
    emulator->Mapper[32] = &MAPPER_G101;
    strcpy(MapperName[32], "Irem G-101");
    emulator->MapperInit[33] = &init_taito_tc0190;
    emulator->Mapper[33] = &MAPPER_TAITO_TC0190;
    strcpy(MapperName[33], "Taito TC0190");
    emulator->MapperInit[64] = &init_tengen_rambo1;
    emulator->Mapper[64] = &MAPPER_TENGEN_RAMBO1;
    strcpy(MapperName[64], "Tengen RAMBO-1");
    emulator->MapperInit[66] = &init_gnrom;
    emulator->Mapper[66] = &MAPPER_GNROM;
    strcpy(MapperName[66], "GNROM");
    emulator->MapperInit[68] = &init_sunsoft4;
    emulator->Mapper[68] = &MAPPER_SUNSOFT4;
    strcpy(MapperName[68], "Sunsoft Mapper #4");
    emulator->MapperInit[69] = &init_fme7;
    emulator->Mapper[69] = &MAPPER_FME7;
    strcpy(MapperName[69], "Sunsoft FME-7");
    emulator->MapperInit[71] = &init_camerica;
    emulator->Mapper[71] = &MAPPER_CAMERICA;
    strcpy(MapperName[71], "Camerica");
    emulator->MapperInit[78] = &init_irem_74hc161_32;
    emulator->Mapper[78] = &MAPPER_IREM_74HC161_32;
    strcpy(MapperName[78], "Irem 74HC161/32");
    emulator->MapperInit[99] = &init_vs;
    emulator->Mapper[99] = &MAPPER_VS;
    strcpy(MapperName[99], "VS UniSystem");
    emulator->MapperInit[225] = &init_supervision;
    emulator->Mapper[225] = &MAPPER_SUPERVISION;
    strcpy(MapperName[225], "SuperVision");
    emulator->MapperInit[231] = &init_nina7;
    emulator->Mapper[231] = &MAPPER_NINA7;
    strcpy(MapperName[231], "NINA-07");
}

void InitMemoryX86(void)
{
  /* Allocate memory */
  if (mmap (RAM_X86, 0x8000,
		  PROT_READ | PROT_WRITE,
		  MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS,
		  -1, 0) <= 0)
    {
      perror ("RAM_X86 mmap");
      exit (1);
    }
  emulator->RAM = RAM_X86;
  if (mmap (ROM_X86, 0x300000,
		  PROT_READ | PROT_WRITE,
                  MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS,
		  -1, 0) <= 0)
    {
      perror ("ROM_X86 mmap");
      exit (1);
    }
  emulator->ROM = ROM_X86;
  if (mmap (CODE_BASE, 0x800000,
		  PROT_READ | PROT_WRITE | PROT_EXEC,
                  MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS,
		  -1, 0) <= 0)
    {
      perror ("CODE_BASE mmap");
      exit (1);
    }
  if (mmap (INT_MAP, 0x400000,
		  PROT_READ | PROT_WRITE,
                  MAP_FIXED | MAP_PRIVATE | MAP_ANONYMOUS,
		  -1, 0) <= 0)
    {
      perror ("INT_MAP mmap");
      exit (1);
    }
  emulator->MapperInit = (void **)MapperInit_X86;
  emulator->Mapper = Mapper_X86;
}

void FreeMemoryX86(void)
{
  int r;
  r = (int) munmap (RAM_X86, 0x8000);
  if (r <= 0)
    {
      perror ("RAM_X86 munmap");
    }
  emulator->RAM = NULL;
  r = (int) munmap (CODE_BASE, 0x800000);
  if (r <= 0)
    {
      perror ("CODE_BASE munmap");
    }
  r = (int) munmap (ROM_X86,0x300000);
  if (r <= 0)
    {
      perror ("ROM_X86 munmap");
    }
  emulator->ROM = NULL;
  r = (int) munmap ((void *) INT_MAP, 0x400000);
  if (r <= 0)
    {
      perror ("INT_MAP munmap");
    }
}

#endif
