/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: emulator_x86.h,v 1.4 2004/07/30 04:25:03 cheako Exp $
 *
 * Description: Deffinition for the X86asm routines.
 */

#ifndef _EMULATOR_X86_H_
#define _EMULATOR_X86_H_

#ifdef HAVE_X86

#define _RAM 0x10000000		/* NES internal 2k RAM */
#define _ROM 0x10010000		/* ROM Image */
#define _CODE_BASE 0x12000000	/* translated code */
#define _INT_MAP 0x13000000	/* translation address map */

#define _ZPMEM _RAM
#define _STACK _ZPMEM+0x100	/* Stack at 0100 */
#define _NVRAM _RAM+0x6000 /* Battery RAM (mapped to 6000 in 6502 address space) */
#define _REG1 _RAM+0x2000	/* ioregs mapped to 2000 in 6502 memory */

#define _VFLAG    VFLAG    /* Store overflow flag */
#define _FLAGS    FLAGS    /* Store 6502 process status reg */
#define _STACKPTR STACKPTR /* Store 6502 stack pointer */
#define _PCR      PCR      /* Store 6502 program counter */
#define _XPC      XPC      /* Translated program counter */
#define _INRET    INRET    /* Input data return value */
#define _CTNI     CTNI     /* Cycles to next interrupt */
#define _SCANPOS  SCANPOS  /* Current scanline position */
#define _VRAM     VRAM     /* Video memory */
#define _VRAMPTR  VRAMPTR  /* address to read/write video memory */
#define _DMOD     DMOD     /* Dest addr to modify */
#define _LASTBANK LASTBANK /* Last memory page code executed in */
#define _MAPTABLE MAPTABLE /* MAPTABLE?? */

#define RAM_X86	((unsigned char *)(_RAM))   /* NES internal 2k RAM */
#define ZPMEM	((unsigned char *)(_ZPMEM)) /* Zero page */
#define STACK	((unsigned char *)(_STACK)) /* stack page */
#define NVRAM	((unsigned char *)(_NVRAM)) /* Battery RAM */
#define ROM_X86	((unsigned char *)(_ROM))   /* ROM Image */
#define CODE_BASE ((unsigned char *)(_CODE_BASE)) /* translated code */
#define INT_MAP ((unsigned int *)_INT_MAP)  /* translation address map */
#undef  REG1				    /* Used by others, in common */
#define REG1	((unsigned char *)(_REG1)   /* REG1?? */

#ifndef _EMULATOR_X86_ASM_S_
extern void START (void);

extern 
  void InitMapperSubsystemX86(void);
extern 
  void InitMemoryX86(void);
extern 
  void FreeMemoryX86(void);
extern 
  void StartX86(void);

extern int DoNMIX86(void);
extern void START(void);
extern void translate(int);

/* Asm linkage */
extern unsigned int	STACKPTR;   /* Store 6502 stack pointer */
extern void	*XPC;               /* Translated program counter */

#endif /* _EMULATOR_X86_ASM_S_ */

#endif /* HAVE_X86 */

#endif /* _EMULATOR_X86_H_ */
