/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: emulator_x86_dynrec.c,v 1.7 2004/05/10 15:00:59 cheako Exp $
 *
 * Description: This is the dynamic recompiler.  This takes 6502 code as
 * input, looks up the opcodes in the translation dictionary, evaluates
 * any expressions found therein, and writes the translated code to
 * *(next_code_alloc++). Pretty simple, huh?
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include "mapper.h"
#include "globals.h"
#include "emulator.h"
#include "emulator_x86.h"

#ifdef HAVE_X86

/* Some declarataions for the asm code */
unsigned int	VFLAG;		/* Store overflow flag */
unsigned int	FLAGS;		/* Store 6502 process status reg */
unsigned int	STACKPTR;	/* Store 6502 stack pointer */
unsigned int	PCR;		/* Store 6502 program counter */
void		*XPC;		/* Translated program counter */
unsigned int	DMOD;		/* Dest addr to modify */
unsigned int	LASTBANK;	/* Last memory page code executed in */

extern int	TRANS_TBL[];
extern void	*INPUT;
extern void	*OUTPUT;
extern void	*U;
extern void	*NMI;

/* x86-specific definitions */
#define NOP 0x90
#define BRK 0xCC

/* forward and external declarations */
void disas(int);
void translate(int);

unsigned char *next_code_alloc = CODE_BASE;

void
translate(int addr)
{
  int saddr;
  unsigned char src;
/*  unsigned int *ptr,*nptr; */
  unsigned int *ptr;
  unsigned char *sptr;
  unsigned char *cptr, *bptr;
  int slen, dlen;
  int x;
  unsigned char m, l, o;
  unsigned char stop = 0;

  XPC = cptr = next_code_alloc;
  if (disassemble)
    {
      printf ("\n[%4x] (%8x) -> %8x\n", addr, (int) (MAPTABLE[addr >> 12] +
                                                     addr), (int)cptr);
      disas (addr);             /* This will output a disassembly of the 6502 code */
    }
  while (!stop)
    {
      saddr = addr;
      INT_MAP[(MAPTABLE[addr >> 12] + addr) - emulator->RAM] = (unsigned int) cptr;
      ptr = (int *) TRANS_TBL;
      while (1)
        {
          src = *(MAPTABLE[addr >> 12] + addr);
          addr++;
          /*printf("%x%x",src>>4,src&0xF);fflush(stdout); */
          if (ptr[src] == 0)
            break;
          if (ptr[src] & 1)
            break;
          ptr = (int *) (ptr[src] + (int) TRANS_TBL);
        }
      if (ptr[src] == 0)
        {
          addr = saddr + 1;
	  /*printf("!%2x-NULL!",src); */
	  if (!ignorebadinstr)
	    *(cptr++) = BRK;
        }
      else
        {
          sptr = (unsigned char *) TRANS_TBL + ptr[src];
          slen = sptr[-1];
          dlen = x = *(sptr++);
          bptr = cptr;
          while (x--)
            *(cptr++) = *(sptr++);
          while (*sptr != 0)
            {
              m = *(sptr++);
              l = *(sptr++);
              o = *(sptr++);
              /*printf("[%c%d %d]",m,o,l);fflush(stdout); */
              switch(m) {
                case '!':
                  stop = 1;
                  break;
                case 'B':
                  bptr[l] = *(MAPTABLE[(saddr + o) >> 12] + (saddr + o));
                  break;
                case 'C':
                  bptr[l] = ~*(MAPTABLE[(saddr + o) >> 12] + (saddr + o));
                  break;
                case 'D':
                  *((unsigned int *) (bptr + l)) = (unsigned int) bptr + l + o;
                  break;
                case 'E':
                  *((int *) (bptr + l)) = *((signed char *) (MAPTABLE[(saddr +
                                                                     o) >>
                                                        12] + (saddr + o)));
                  break;
                case 'Z':
                  *((unsigned int *) (bptr + l)) = (unsigned int) ZPMEM +
                    *(MAPTABLE[(saddr
                              +
                              o)
                             >>
                             12]
                    +
                    saddr
                    + o);
                  break;
                case 'A':
                  *((unsigned int *) (bptr + l)) =
		   (unsigned int) emulator->RAM + *(MAPTABLE[(saddr
                              +
                              o)
                             >>
                             12]
                    +
                    saddr
                    + o)
                  + ((*(MAPTABLE[(saddr + o + 1) >> 12] + saddr + o + 1)) << 8);
                  break;
                case 'L':
                  *((unsigned int *) (bptr + l)) =
		   	(unsigned int) emulator->RAM;
                  break;
                case 'W':
                  *((unsigned short *) (bptr + l)) = *(MAPTABLE[(saddr + o) >>
                                                          12] + saddr + o) +
                    ((*(MAPTABLE[(saddr
                                +
                                o
                                +
                                1)
                               >>
                               12]
                      +
                      saddr
                      +
                      o
                      +
                      1))
                     << 8);
                  break;
                case 'X':
                  *((unsigned int *) (bptr + l)) = (unsigned int) (MAPTABLE +
                                                         ((*(MAPTABLE[(saddr
                                                                       +
                                                                       o
                                                                       +
                                                                       1)
                                                                      >>
                                                                      12]
                                                             +
                                                             saddr
                                                             +
                                                             o
                                                             +
                                                             1))
                                                          >> 4));
                  break;
                case 'M':
                  *((unsigned int *) (bptr + l)) = (unsigned int) (MAPTABLE);
                  break;
                case 'T':
                  *((unsigned int *) (bptr + l)) = (unsigned int) STACK;
                  break;
                case 'P':
                  *((unsigned short *) (bptr + l)) = saddr + o;
                  break;
                case 'R':
                  *((int *) (bptr + l)) = *((signed char *) (MAPTABLE[(saddr +
                                                                     o) >>
                                                           12] + saddr + o))
                    + saddr + o + 1;
                  break;
                case 'J':
                  *((unsigned int *) (bptr + l)) = *(MAPTABLE[(saddr + o) >>
                                                          12] + saddr + o) +
                    ((*(MAPTABLE[(saddr
                                +
                                o
                                +
                                1)
                               >>
                               12]
                      +
                      saddr
                      + o
                      +
                      1))
                     << 8);
                  break;
                case 'S':
                  *((void **) (bptr + l)) = &STACKPTR;
                  break;
                case 'V':
                  *((void **) (bptr + l)) = &VFLAG;
                  break;
                case 'F':
                  *((void **) (bptr + l)) = &FLAGS;
                  break;
                case 'I':
                  *((int *) (bptr + l)) = (int) (&INPUT) - (int) (bptr + l) - 4;
                  break;
                case 'O':
                  *((int *) (bptr + l)) = (int) (&OUTPUT) - (int) (bptr + l) 
                      - 4;
                  break;
                case 'U':
                  *((int *) (bptr + l)) = (int) (&U) - (int) (bptr + l) - 4;
                  break;
                case 'N':
                  *((int *) (bptr + l)) = (int) (&NMI) - (int) (bptr + l) - 4;
                  break;
                case 'Y':
                  *((int *) (bptr + l)) = (int)
		   (emulator->Mapper[MAPPERNUMBER]) - (int)
                    (bptr
                     +
                     l) - 4;
                  break;
                case '>':
                  bptr[l] += (((*((signed char *) (MAPTABLE[(saddr + o) >> 12]
                                                 + saddr + o)) + saddr + o +
                              1) & 0xFF00) != ((saddr + o + 1) & 0xFF00));
                  break;
                case '^':
                  bptr[l] = ignorebadinstr ? NOP : BRK;
                  break;
              }
            }
          addr = saddr + slen;
        }
    }
  while (((int) cptr & 15) != 0)
    *(cptr++) = NOP;
  next_code_alloc = cptr;
}

#endif /* HAVE_X86 */
