/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: renderer.h,v 1.16 2004/05/11 08:19:33 cheako Exp $
 *
 * Description: Abstract renderer interface
 */

#ifndef X_DISPLAY_MISSING
#define HAVE_X 1
#endif /* !defined(X_DISPLAY_MISSING) */

#include <sys/types.h>
#ifdef HAVE_WINSOCK_H
# include <winsock.h>
#endif

#include "types.h"

#ifdef HAVE_LIBGII
#ifdef HAVE_GGI_GII_H
#ifdef HAVE_LIBGGI
#ifdef HAVE_GGI_GGI_H
#define HAVE_GGI 1
#endif
#endif
#endif
#endif

#ifdef HAVE_LIBW
#ifdef HAVE_WLIB_H
#define HAVE_W 1
#endif
#endif

#ifdef HAVE_LIBSDL
#ifdef HAVE_SDL_SDL_H
#define HAVE_SDL 1
#endif
#endif

/* renderer flags */
#define RENDERER_OLD		1
#define RENDERER_DIFF		2
#define RENDERER_NON_X11	4

struct Renderer {
     char *name, *fullname, *controlname;
     int _flags;
     int (*InitDisplay)(int argc, char **argv);
     void (*UpdateDisplay)(void);
     void (*UpdateColors)(void);
     void (*ToggleFullScreen)(void);
     void (*SaveScreenshot)(void);
};

/* the currently selected renderer */
extern struct Renderer *renderer;

/* table of renderers, terminated by { 0, ... } */
extern struct Renderer renderers[];

/* global renderer parameters */
extern struct RendererConfig {
     int	inroot;
     int	sticky_keys;
     char	*geometry;
     char	*display_id;
} renderer_config;

/* global renderer data */
extern struct RendererData {
     struct {
         time_t sec;
         suseconds_t usec;
     } basetime;
     int	pause_display;
     int	pause_program;
#define PAUSED (renderer_data.pause_display || renderer_data.pause_program)
} renderer_data;

extern int InitDisplayAuto(int argc, char **argv);
extern int InitDisplayNone(int argc, char **argv);
extern void UpdateColorsNone(void);
extern void UpdateDisplayNone(void);

/* only the no-op renderer `none' is universally known */
extern int	InitDisplayNone(int argc, char **argv);
extern void	UpdateColorsNone(void);
extern void	UpdateDisplayNone(void);
