/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: renderer_sdl.c,v 1.6 2004/07/30 04:25:03 cheako Exp $
 *
 * Description: This file handles the I/O subsystem when using SDL.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_FEATURES_H
#include <features.h>
#endif				/* HAVE_FEATRES_H */

#include "consts.h"
#include "globals.h"
#include "mapper.h"
#include "renderer.h"
#include "sounder.h"
#include "controller.h"
#include "emulator.h"
#include "renderer_sdl.h"
#include "renderer_util.h"

#ifdef HAVE_LIBSDL

#include <SDL/SDL.h>
#include <SDL/SDL_endian.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

char    *screenshotfile;
static int     screenshotnumber = 0;

#define BPP 16

SDL_Surface *screen;
unsigned int paletteSDL[64];

/* ============================ SaveScreenshotSDL () ======================= */
void SaveScreenshotSDL(void)
{
    screenshot_check_filename (".bmp", screenshotfile, &screenshotnumber);
    SDL_SaveBMP(screen, screenshotfile);
}

/* ============================ InitDisplaySDL () ========================== */
int InitDisplaySDL(int argc, char **argv)
{
    int i;
    if (magstep < 1)
	magstep = 1;
    if (magstep > maxsize) {
	fprintf(stderr, "Warning: Enlargement factor %d is too large!\n",
		magstep);
	magstep = maxsize;
    }
    bpp = BPP;
    bytes_per_line = 256 * magstep * BPP / 8;
    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
	fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
	exit(1);
    }
    atexit (SDL_Quit);
    if ((screen =
	 SDL_SetVideoMode(256 * magstep, 240 * magstep, BPP,
			  SDL_SWSURFACE)) == NULL) {
	fprintf(stderr, "Unable to set 256x240 video: %s\n",
		SDL_GetError());
	exit(1);
    }
    rfb = fb = screen->pixels;
    for (i = 0; i < 64; i++) {
	unsigned char r, g, b;
	r = (NES_palette[i] >> 16 >> 3) & 0x1f;
	g = (NES_palette[i] >> 8 >> 2) & 0x3f;
	b = (NES_palette[i] >> 3) & 0x1f;
	paletteSDL[i] = (r << 11) | (g << 5) | (b);
    }
    fbinit();
    screenshot_init(".bmp", &screenshotfile, &screenshotnumber);
    set_renderer_basetime();
    return 0;
}

/* ============================ UpdateDisplaySDL () ======================== */
void UpdateDisplaySDL(void)
{
    unsigned static int frame;

    sounder->UpdateAudio();
    frame++;
    frame = resynchronize(frame);
    drawimage(PBL);
    if (!frameskip) {
	SDL_UpdateRect(screen, 0, 0, 0, 0);
    }
}

/* ============================ UpdateColorsSDL () ========================= */
void UpdateColorsSDL(void)
{
    int i;
    palette[24] = paletteSDL[VRAM[0x3f00] & 63];
    for (i = 0; i < 24; i++)
	palette[i] = paletteSDL[VRAM[0x3f01 + i + (i / 3)] & 63];
}

/* ============================ ToggleFullScreenSDL () ===================== */ 
void ToggleFullScreenSDL(void)
{
    SDL_WM_ToggleFullScreen(screen);
}

#endif				/* HAVE_LIBSDL */
