/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: renderer_util.c,v 1.7 2004/07/28 21:15:42 cheako Exp $
 *
 * Description: Shared code used by all of the tuxnes renderers.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#include <dirent.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <globals.h>
#include <stdio.h>
#include <renderer.h>
#include <unistd.h>

#ifdef HAVE_GETTIMEOFDAY
# include <sys/time.h>
# include <time.h>
#endif

#ifdef HAVE_FTIME
# include <sys/timeb.h>
#endif

#include "renderer_util.h"

/* ============================ screenshot_init () ========================= */
void screenshot_init(char *ext, char **ssfile, int *ssnumber)
{
    DIR *dir;
    struct dirent *dirp;
    int currentnumber = 0;
    /* Allocate space for (tuxnesdir) + (basefilename-snap-xxxx.ext) + ('\0') */
    if ((*ssfile = (char *) malloc(strlen(tuxnesdir) +
				   strlen(basefilename) + 1 + 4 +
				   1 + 4 + 1 + 3 + 1)) == NULL) {
	perror("malloc");
	exit(1);
    }
    sprintf(*ssfile, "%s-snap-", basefilename);
    /* open the screenshot directory */
    if ((dir = opendir(tuxnesdir)) == NULL) {
	return;
    }
    /* iterate through the files and establish the starting screenshot number */
    while ((dirp = readdir(dir)) != NULL) {
	if ((strlen(dirp->d_name) >= 8)
	    && (strncmp(dirp->d_name, *ssfile, strlen(*ssfile))
		== 0)
	    &&
	    (strncmp
	     (dirp->d_name + strlen(dirp->d_name) - 4, ext,
	      strlen(ext)) == 0)) {
	    dirp->d_name[strlen(dirp->d_name) - 4] = '\0';
	    if ((currentnumber =
		 atoi(dirp->d_name + strlen(*ssfile))) > *ssnumber) {
		*ssnumber = currentnumber;
	    }
	}
    }
    if (++*ssnumber > 9999) {
	*ssnumber = 0;
    }
    closedir(dir);
}

/* ============================ screenshot_check_filename () =============== */
void screenshot_check_filename(char *ext, char *ssfile, int *ssnumber)
{
    /* make sure we don't over-write screenshots written by a concurrent TuxNES process */
    {
	struct stat buf[1];
	do
	    sprintf(ssfile, "%s%s-snap-%04u%s", tuxnesdir,
		    basefilename, *ssnumber++, ext);
	while ((!stat(ssfile, buf)) && !(*ssnumber > 9999));
    }
    if (*ssnumber > 9999)
	*ssnumber = 0;
}


/* ============================ get_time () =================== */
static void get_time(time_t *sec, suseconds_t *usec)
{
#ifdef HAVE_GETTIMEOFDAY
  struct timeval tv;

  gettimeofday(&tv, NULL);
  *sec = tv.tv_sec;
  *usec = tv.tv_usec;
#elif defined HAVE_FTIME
  struct timeb tp;

  ftime(&tp);
  *sec = tp.time;
  *usec = (suseconds_t)tp.millitm * 1000;
#else
#  error Neither gettimeofday() nor ftime() is present. \
         Report this error to the tuxnes developers.
#endif
}

/* ============================ set_renderer_basetime () =================== */
void set_renderer_basetime(void)
{
    get_time(&renderer_data.basetime.sec, &renderer_data.basetime.usec);
}

/* ============================ resynchronize () =========================== */
int resynchronize(int frame)
{
  time_t now_sec;
  suseconds_t now_usec, usec;

  frameskip = desync = 0;

#define FRAMES_PER_SECOND 60 /* NTSC */

  usec = 1000000 * frame / FRAMES_PER_SECOND;

  /* Note: if we switch speeds while running, reset frame and basetime. */
  if (halfspeed) {
    usec *= 2;
  } else if (doublespeed > 1) {
    usec /= doublespeed;
  }

  get_time(&now_sec, &now_usec);

  usec += (1000000 * (renderer_data.basetime.sec - now_sec)
	   + (renderer_data.basetime.usec - now_usec));

  if (usec > 0) {
    if ( frame > 1800 ) /* Every 30 seconds */
       {
	  frame = 0;
	  renderer_data.basetime.sec = now_sec;
	  renderer_data.basetime.usec = now_usec;
       }
    usleep(usec);
  } else if (usec < -2 * 1000000 / FRAMES_PER_SECOND) {
    /* If we're more than a frame behind, skip a frame. */
    frameskip = 1;

    if (usec < -20 * 1000000 / FRAMES_PER_SECOND) {
      /* If we're more than 20 frames behind, give up on syncing. */
      desync = 1;
    }
  }
  return frame;
}
