/*
 * This file is part of the TuxNES project codebase.
 *
 * Please see the README and COPYING files for more information regarding
 * this project.
 *
 * $Id: sounder_sdl.c,v 1.8 2004/08/23 23:54:58 cheako Exp $
 *
 * Description: This file handles the sound subsystem when using SDL.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdlib.h>

#ifdef HAVE_FEATURES_H
#include <features.h>
#endif				/* HAVE_FEATRES_H */

#include "globals.h"
#include "sound.h"
#include "sounder.h"
#include "sounder_sdl.h"

#ifdef HAVE_LIBSDL
#include <SDL/SDL.h>
#include <string.h>

/* table of sample formats, terminated by { 0, ... } */
struct SampleFormat SampleFormatsSDL[] = {
     { "8",      "8-bit unsigned",
       AUDIO_U8, 1, 0, 0 },
     { "8s",     "8-bit signed",
       AUDIO_S8, 1, 1, 0 },
     { "16",     "16-bit signed",
       AUDIO_S16SYS, 2, 1, 0 },
     { "16u",    "16-bit unsigned",
       AUDIO_U16SYS, 2, 0, 0 },
     { "le16",   "16-bit signed (little-endian)",
       AUDIO_S16LSB, 2, 1, 1 },
     { "le16u",  "16-bit unsigned (little-endian)",
       AUDIO_U16LSB, 2, 0, 1 },
     { "be16",   "16-bit signed (big-endian)",
       AUDIO_S16MSB, 2, 1, 2 },
     { "be16u",  "16-bit unsigned (big-endian)",
       AUDIO_U16MSB, 2, 0, 2 },
     { 0, 0, 0, 0, 0, 0 }
};

static void mixaudio(void *unused, Uint8 * stream, int len)
{
   /* FIXME len != samples_per_vsync */
   /* Subjest fifo/ring type audio_buffer */
    memcpy (stream, audio_buffer, samples_per_vsync);
}

int InitAudioSDL(int argc, char **argv)
{
    SDL_AudioSpec wanted;
    InitAudioCore (argc, argv); /* Need samples_per_vsync set before */
                                /* SDL_OpenAudio. */
    /* Set the audio format */
#if 1
    wanted.freq = 44100;
    wanted.format = AUDIO_S8;
    wanted.channels = 1;
#else
    wanted.freq = sound_config.audiorate;
    wanted.format = sample_format->number;
    wanted.channels = (sound_config.audiostereo != 0) + 1 ;
#endif
    wanted.samples = samples_per_vsync;
    wanted.callback = mixaudio;
    wanted.userdata = NULL;
    if (SDL_Init(SDL_INIT_AUDIO) < 0) {
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        exit(1);
    }
    if (SDL_OpenAudio(&wanted, NULL) < 0) {
	fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
	return (1);
    }
    SDL_PauseAudio (0);
    return 0;
}

void UpdateAudioSDL (void)
{
    SDL_LockAudio();
    ProgressiveSoundUpdate (1);
    SDL_UnlockAudio();
}

void ShutdownAudioSDL (void)
{
    SDL_CloseAudio();
}

#endif				/* HAVE_LIBSDL */
